@@ -58,9 +58,9 @@
 
 #include "Screen.h"
 
-#define MAX_SYNOPSIS_STRINGS 500  
+#define MAX_SYNOPSIS_STRINGS 500
 
-//declare variables local to this file.  
+//declare variables local to this file.
 static const char *synopsisSYNOPSIS[MAX_SYNOPSIS_STRINGS];
 
 void InitializeSynopsis()
@@ -76,7 +76,7 @@ void InitializeSynopsis()
 
     // Open or close a window or texture:
     synopsis[i++] = "\n% Open or close a window or texture:";
-    synopsis[i++] = "[windowPtr,rect]=Screen('OpenWindow',windowPtrOrScreenNumber [,color] [,rect] [,pixelSize] [,numberOfBuffers] [,stereomode] [,multisample][,imagingmode][,specialFlags][,clientRect]);";
+    synopsis[i++] = "[windowPtr,rect]=Screen('OpenWindow',windowPtrOrScreenNumber [,color] [,rect] [,pixelSize] [,numberOfBuffers] [,stereomode] [,multisample][,imagingmode][,specialFlags][,clientRect][,fbOverrideRect]);";
     synopsis[i++] = "[windowPtr,rect]=Screen('OpenOffscreenWindow',windowPtrOrScreenNumber [,color] [,rect] [,pixelSize] [,specialFlags] [,multiSample]);";
     synopsis[i++] = "textureIndex=Screen('MakeTexture', WindowIndex, imageMatrix [, optimizeForDrawAngle=0] [, specialFlags=0] [, floatprecision=0] [, textureOrientation=0] [, textureShader=0]);";
     synopsis[i++] = "oldParams = Screen('PanelFitter', windowPtr [, newParams]);";
@@ -112,7 +112,7 @@ void InitializeSynopsis()
     synopsis[i++] = "oldTextSize=Screen('TextSize', windowPtr [,textSize]);";
     synopsis[i++] = "oldStyle=Screen('TextStyle', windowPtr [,style]);";
     synopsis[i++] = "[oldFontName,oldFontNumber,oldTextStyle]=Screen('TextFont', windowPtr [,fontNameOrNumber][,textStyle]);";
-    synopsis[i++] = "[normBoundsRect, offsetBoundsRect, textHeight]= Screen('TextBounds', windowPtr, text [,x] [,y] [,yPositionIsBaseline] [,swapTextDirection]);";
+    synopsis[i++] = "[normBoundsRect, offsetBoundsRect, textHeight, xAdvance] = Screen('TextBounds', windowPtr, text [,x] [,y] [,yPositionIsBaseline] [,swapTextDirection]);";
     synopsis[i++] = "[newX, newY, textHeight]=Screen('DrawText', windowPtr, text [,x] [,y] [,color] [,backgroundColor] [,yPositionIsBaseline] [,swapTextDirection]);";
     synopsis[i++] = "oldTextColor=Screen('TextColor', windowPtr [,colorVector]);";
     synopsis[i++] = "oldTextBackgroundColor=Screen('TextBackgroundColor', windowPtr [,colorVector]);";
@@ -167,6 +167,7 @@ void InitializeSynopsis()
     synopsis[i++] = "resolutions=Screen('Resolutions', screenNumber);";
     synopsis[i++] = "oldResolution=Screen('Resolution', screenNumber [, newwidth] [, newheight] [, newHz] [, newPixelSize] [, specialMode]);";
     synopsis[i++] = "oldSettings = Screen('ConfigureDisplay', setting, screenNumber, outputId [, newwidth][, newheight][, newHz][, newX][, newY]);";
+    synopsis[i++] = "Screen('ConstrainCursor', windowIndex, addConstraint [, rect]);";
 
     // Get and set information about the environment, computer, and video card (i.e. screen):
     synopsis[i++] = "\n% Get/set details of environment, computer, and video card (i.e. screen):";
@@ -191,6 +192,7 @@ void InitializeSynopsis()
     synopsis[i++] = "Screen('HideCursorHelper', windowPntr [, mouseIndex]);";
     synopsis[i++] = "Screen('ShowCursorHelper', windowPntr [, cursorshapeid][, mouseIndex]);";
     synopsis[i++] = "Screen('SetMouseHelper', windowPntrOrScreenNumber, x, y [, mouseIndex][, detachFromMouse]);";
+    synopsis[i++] = "Screen('SetMouseHelper', windowPntrOrScreenNumber, x, y [, mouseIndex][, detachFromMouse]);";
 
     // Internal testing of Screen
     synopsis[i++] = "\n% Internal testing of Screen";
